// MCP23017 en PCF8575 routines

#include <Wire.h>            //tbv MCP23017 of PCF8575
byte   MCPioA[7];            //ioByte A in MCP's
byte   MCPioB[7];            //ioByte B in MCP's
int    MCPtype[7];           //0 = not ready, 1 = MCP, 2 = PCF
byte testdata = 0;



//------------------
void MCPinit(byte adres, byte Afunctie, byte Bfunctie) { 
 // Initialiseren van alle pinnen van de MCP23017 of PCF8575
 // Er wordt bepaald of de IOchip een MPC of een PCG is, dit wordt
 // vastgehouden in de array MCPtype.
 // Adres opgeven en pinnen instellen voor input of output
 // NB: dit hoeft maar eenmalig, of als pinnen aan andere functie krijgen.
 // Het adres van de MCP of PCF is hex 20tm27, afhankelijk van adresjumpers
 //     jumper A0 telt voor 1, A1 voor 2 en A2 voor 4.
 // Afunctie en Bfunctie zijn de functiebits voor de poorten A en B
 // Als een pin output moet zijn staat de bit uit
 // Als een pin input moet zijn staat de bit aan
 // Bijv:   MCPinit( 0x20, B00000001, B11111110) 
 //               poort A pin 7 tm 1 output, pin0 input
 //               poort B pin 7 tm 1 input,  pin0 output
 // Dit kan je ook opgeven als:
 //         MCPinit( 0x20, 1, 254)
 byte byta = 0;
 byte bytb = 0; 
 MCPtype[adres-32] = 0;           //not ready 
 Wire.begin();                    //create Wire object
 Wire.beginTransmission(adres);   //begin communicatie met MCP
 Wire.write(byte(0));             //select IODIRA register, ports 0-7
 Wire.write(Afunctie);            //alle ports 0-7 outputs
 Wire.endTransmission();          //stop commmunicatie met MCP
 Wire.beginTransmission(adres);   //begin communicatie met MCP
 Wire.write(0x01);                //select IODIRB register, ports 8-15
 Wire.write(Bfunctie);            //alle ports 8-15 outputs
 Wire.endTransmission();          //stop commmunicatie met MCP
 Wire.beginTransmission(adres);    //begin communicatie met MCP
 Wire.write(0x12);                 //select GPIOA pins 0-7
 Wire.write(0x1);                  //set pin 0 op ioA
 Wire.endTransmission();           //stop commmunicatie met MCP
 Wire.beginTransmission(adres);    //begin communicatie met MCP
 Wire.write(0x12);                 //GPIOA 
 Wire.endTransmission();           //stop commmunicatie met MCP 
 Wire.requestFrom(adres, (uint8_t)1);
 byta = Wire.read();              //haal huidige pinwaarde op
 if(byta !=1){
  MCPstatus = 9;                  //unit niet ready
 }else{
  MCPstatus = 1;                  //OK, ready  
  Wire.beginTransmission(adres);  //begin communicatie met MCP
  Wire.write(0x12);               //select GPIOA pins 0-7
  Wire.write(0x0);                //reset ioA
  Wire.endTransmission();         //stop commmunicatie met MCP
  MCPtype[adres-32] = 1;          //MCP ready                     
 } 
 if (MCPtype[adres-32] == 0){     //MCP not ready, probeer of PCF aanwezig is
  Wire.beginTransmission(adres);  //begin communicatie met PCF
  Wire.write(byte(B00000000));    //reset low order byte all ports 
  Wire.write(byte(B00000000));    //reset high order byte all ports
  testdata = Wire.endTransmission();     //zend data
  if (testdata == 0) {            //returncode 0 = OK
   MCPstatus = 1;                 //OK, ready 
   MCPtype[adres-32] = 2;         //PCF ready 
  }else{
   MCPstatus = 9;                 //not ready                
  }
 }
 MCPioA[adres-32] = 0;            //ioAbyte(MCP) of low byte(PCF)
 MCPioB[adres-32] = 0;            //ioBbyte(MCP) of high byte(PCF)
}

void  MCPsetPin(byte  adres, byte pinnummer ){
 // Aanzetten van een pin van de MCP23017
 // pinnummers van groep A en B zijn doorgenummerd als 0 tm 15
 double pindouble = 0;
 byte pinwaarde = 0;
 byte oldpinwaarde = 0;
 if (MCPtype[adres-32] == 1){       //MCP
  if (pinnummer < 8){
   pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioA[adres-32] | pinwaarde);
   MCPioA[adres-32] = pinwaarde; 
   Wire.beginTransmission(adres);     //begin communicatie met MCP
   Wire.write(0x12);                  //GPIOA 
   Wire.write(pinwaarde);             //set pin 
   Wire.endTransmission();            //stop commmunicatie met MCP 
  }else{  
   pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioB[adres-32]  | pinwaarde);
   MCPioB[adres-32] = pinwaarde;
   Wire.beginTransmission(adres);     //begin communicatie met MCP
   Wire.write(0x13);                  //GPIOB 
   Wire.write(pinwaarde);             //set pin 
   Wire.endTransmission();            //stop commmunicatie met MCP 
  } 
 }else{                               //PCF  
  if (pinnummer < 8){
   pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioA[adres-32] | pinwaarde);
   MCPioA[adres-32] = pinwaarde; 
  }else{  
   pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioB[adres-32]  | pinwaarde);
   MCPioB[adres-32] = pinwaarde;
  } 
  Wire.beginTransmission(adres);     //begin communicatie met PCF
  Wire.write (MCPioA[adres-32]) ;    //low order byte 
  Wire.write (MCPioB[adres-32]) ;    //hig order byte 
  testdata = Wire.endTransmission(); //stop commmunicatie met PCF 
  Serial.println("RC1 " + String(testdata));
 }
}

void  MCPresetPin(byte  adres, byte pinnummer ){
 // Aanzetten van een pin van de MCP23017
 // pinnummers van groep A en B zijn doorgenummerd als 0 tm 15
 double pindouble = 0;
 byte pinwaarde = 0;
 byte oldpinwaarde = 0;
 if (MCPtype[adres-32] == 1){       //MCP
  if (pinnummer < 8){
   pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioA[adres-32] & ~pinwaarde);
   MCPioA[adres-32] = pinwaarde; 
   Wire.beginTransmission(adres);     //begin communicatie met MCP
   Wire.write(0x12);                  //GPIOA 
   Wire.write(pinwaarde);             //set pin 
   Wire.endTransmission();            //stop commmunicatie met MCP 
  }else{  
   pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioB[adres-32]  & ~pinwaarde);
   MCPioB[adres-32] = pinwaarde;
   Wire.beginTransmission(adres);     //begin communicatie met MCP
   Wire.write(0x13);                  //GPIOB 
   Wire.write(pinwaarde);             //set pin  
   Wire.endTransmission();            //stop commmunicatie met MCP 
  }
 }else{                               //PCF
  if (pinnummer < 8){
   pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioA[adres-32] & ~pinwaarde);
   MCPioA[adres-32] = pinwaarde; 
  }else{  
   pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioB[adres-32] & ~pinwaarde);
   MCPioB[adres-32] = pinwaarde;
  } 
  Wire.beginTransmission(adres);     //begin communicatie met PCF
  Wire.write (MCPioA[adres-32]) ;   //low order byte
  Wire.write (MCPioB[adres-32]) ;   //hig order byte 
  Wire.endTransmission();            //stop commmunicatie met PCF
 }
}

void  MCPflipPin(byte  adres, byte pinnummer ){
 // Aanzetten van een pin van de MCP23017
 // pinnummers van groep A en B zijn doorgenummerd als 0 tm 15
 double pindouble = 0;
 byte pinwaarde = 0;
 byte oldpinwaarde = 0;
 if (MCPtype[adres-32] == 1){          //MCP
  if (pinnummer < 8){
   pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioA[adres-32] ^ pinwaarde);
   MCPioA[adres-32] = pinwaarde; 
   Wire.beginTransmission(adres);     //begin communicatie met MCP
   Wire.write(0x12);                  //GPIOA 
   Wire.write(pinwaarde);             //set pin 
   Wire.endTransmission();            //stop commmunicatie met MCP 
  }else{  
   pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioB[adres-32]  ^ pinwaarde);
   MCPioB[adres-32] = pinwaarde; 
   Wire.beginTransmission(adres);     //begin communicatie met MCP
   Wire.write(0x13);                  //GPIOB 
   Wire.write(pinwaarde);             //set pin 
   Wire.endTransmission();            //stop commmunicatie met MCP 
  }
 }else{                              //PCF
  if (pinnummer < 8){
   pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioA[adres-32] ^ pinwaarde);
   MCPioA[adres-32] = pinwaarde; 
  }else{  
   pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
   pinwaarde = pindouble; 
   pinwaarde = (MCPioB[adres-32] ^ pinwaarde);
   MCPioB[adres-32] = pinwaarde;
  } 
  Wire.beginTransmission(adres);     //begin communicatie met PCF
  Wire.write (MCPioA[adres-32]) ;   //low order byte 
  Wire.write (MCPioB[adres-32]) ;   //hig order byte 
  Wire.endTransmission();            //stop commmunicatie met PCF
 }   
}

void  MCPsetAllpins(byte  adres, byte  pinA0t7, byte  pinB0t7){
 // Zetten van alle pinnen van de MCP23017 tegelijk
 // Bij pinA0t7 en pinB0t7 de som van de pinwaarden opgeven
 // pin0 heeft waarde 1
 // pin1 heeft waarde 2
 // pin2 heeft waarde 4
 // pin3 heeft waarde 8
 // pin4 heeft waarde 16
 // pin5 heeft waarde 32
 // pin6 heeft waarde 64
 // pin7 heeft waarde 128 
 // BIJV: - alle pinnen uit = MCPsetAllpins(0x20, 0,0)
 //       - alle pinnen aan = MCPsetAllpins(0x20, 255, 255)
 //       - pinnen A0 en A5 aan en B1 en B6 aan = MCPsetAllpins(0x20, 33, 66)
 //                de andere pinnen staan dan uit
 // Je kan de bits van de pinnen ook binair opgeven. 
 // Bijv: MCPsetAllpins(0x20, B00000100 , B10000000)
 //       poort A: pin 2 is aan, poort B pin 7 is aan 
 if (MCPtype[adres-32] == 1){       //MCP
  Wire.beginTransmission(adres);    //begin communicatie met MCP
  Wire.write(0x12);                 //select GPIOA pins 0-7
  Wire.write(pinA0t7);              //set pins ioA
  MCPioA[adres-32] = pinA0t7;
  Wire.endTransmission();           //stop commmunicatie met MCP
  Wire.beginTransmission(adres);    //begin communicatie met MCP
  Wire.write(0x13);                 //select GPIOB pins 8-15
  Wire.write(pinB0t7);              //set pins ioB
  MCPioB[adres-32] = pinB0t7;
  Wire.endTransmission();           //stop commmunicatie met MCP
 }else{                             //PCF
  Wire.beginTransmission(adres);    //begin communicatie met PCF
  Wire.write(pinA0t7);             //set lowbyte pins 
  MCPioA[adres-32] = pinA0t7;
  Wire.write(pinB0t7);             //set highbyte pins 
  MCPioB[adres-32] = pinB0t7;
  Wire.endTransmission();           //stop commmunicatie met PCF
 }
}

bool  MCPreadPin(byte adres, byte pinnummer){
 // Uitlezen van een pin van de MCP23017
 // NB, ook de status van outputpinnen kan worden uitgelezen
 // pinnummers van groep A en B zijn doorgenummerd als 0 tm 15
 double pindouble = 0;
 byte pinwaarde = 0;
 byte oldpinwaarde = 0;
 byte GPIO = 0;
 if (pinnummer < 8){
  GPIO = 0x12;                       //select GPIOA pins 0-7
  pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde 
 }else{  
  GPIO = 0x13;                       //select GPIOB pins 0-7
  pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
 } 
 if (MCPtype[adres-32] == 1){        //MCP
  Wire.beginTransmission(adres);     //begin communicatie met MCP
  Wire.write(GPIO);                  //GPIOA of GPIOB 
  Wire.endTransmission();            //stop commmunicatie met MCP 
  Wire.requestFrom(adres, (uint8_t)1);
  oldpinwaarde = Wire.read();        //haal huidige pinwaarde op 
 }else{                              //PCF
  Wire.beginTransmission(adres);     //begin communicatie met PCF
  oldpinwaarde = Wire.read();        //haal low byte huidige pinwaarde op 
  if (pinnummer < 8){
   oldpinwaarde = Wire.read();       //haal high byte huidige pinwaarde op 
  }
  Wire.endTransmission();            //stop commmunicatie met PCF 
 }
 pinwaarde = pindouble;
 pinwaarde = (oldpinwaarde & pinwaarde);
 return pinwaarde; 
}

void MCPreadAllpins(byte adres, byte &byteA, byte &byteB){
 // Uitlezen van alle pinnen van de MCP23017
 // NB, ook de status van outputpinnen kan worden uitgelezen
 // byteA en byteB bevatten de som van de pinwaarden van de pinnen
 if (MCPtype[adres-32] == 1){        //MCP
  Wire.beginTransmission(adres);     //begin communicatie met MCP
  Wire.write(0x12);                  //GPIOA of GPIOB 
  Wire.endTransmission();            //stop commmunicatie met MCP 
  Wire.requestFrom(adres, (uint8_t)1);  //een byte   
  byteA = Wire.read();               //haal huidige pinwaarden op 
  Wire.beginTransmission(adres);     //begin communicatie met MCP
  Wire.write(0x13);                  //GPIOA of GPIOB 
  Wire.endTransmission();            //stop commmunicatie met MCP 
  Wire.requestFrom(adres, (uint8_t)1);  //een byte
  byteB = Wire.read();               //haal huidige pinwaarden op  
 }else{
  Wire.beginTransmission(adres);     //begin communicatie met PCF
  byteA = Wire.read();               //haal low order byte op 
  byteB = Wire.read();               //haal high order byte op  
  Wire.endTransmission();            //stop commmunicatie met PCF 
 }
}
