#include <Wire.h>                  //tbv MCP23017 
void  MCPsetAllpins(byte  adres, byte  pinA0t7, byte  pinB0t7){
 // Zetten van alle pinnen van de MCP23017 tegelijk
 // Bij pinA0t7 en pinB0t7 de som van de pinwaarden opgeven
 // pin0 heeft waarde 1
 // pin1 heeft waarde 2
 // pin2 heeft waarde 4
 // pin3 heeft waarde 8
 // pin4 heeft waarde 16
 // pin5 heeft waarde 32
 // pin6 heeft waarde 64
 // pin7 heeft waarde 128 
 // BIJV: - alle pinnen uit = MCPsetAllpins(0x20, 0,0)
 //       - alle pinnen aan = MCPsetAllpins(0x20, 255, 255)
 //       - pinnen A0 en A5 aan en B1 en B6 aan = MCPsetAllpins(0x20, 33, 66)
 //                de andere pinnen staan dan uit
 // Je kan de bits van de pinnen ook binair opgeven. 
 // Bijv: MCPsetAllpins(0x20, B00000100 , B10000000)
 //       poort A: pin 2 is aan, poort B pin 7 is aan 
 Wire.beginTransmission(adres);    //begin communicatie met MCP
 Wire.write(0x12);                 //select GPIOA pins 0-7
 Wire.write(pinA0t7);              //set pins 
 Wire.endTransmission();           //stop commmunicatie met MCP
 Wire.beginTransmission(adres);    //begin communicatie met MCP
 Wire.write(0x13);                 //select GPIOB pins 8-15
 Wire.write(pinB0t7);              //set pin 8
 Wire.endTransmission();           //stop commmunicatie met MCP
}

void  MCPsetPin(byte  adres, byte pinnummer ){
 // Aanzetten van een pin van de MCP23017
 // pinnummers van groep A en B zijn doorgenummerd als 0 tm 15
 byte GPIO = 0;
 double pindouble = 0;
 byte pinwaarde = 0;
 byte oldpinwaarde = 0;
 if (pinnummer < 8){
  GPIO = 0x12;                           //select GPIOA pins 0-7
  pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde
 }else{  
  GPIO = 0x13;                           //select GPIOB pins 0-7
  pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
 }  
 Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(GPIO);                  //GPIOA of GPIOB 
 Wire.endTransmission();            //stop commmunicatie met MCP 
 Wire.requestFrom(adres, 0x1);
 oldpinwaarde = Wire.read();        //haal huidige pinwaarde op 
 pinwaarde = pindouble; 
 pinwaarde = (oldpinwaarde | pinwaarde);
 Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(GPIO);                  //GPIOA of GPIOB 
 Wire.write(pinwaarde);             //set pin 
 Wire.endTransmission();            //stop commmunicatie met MCP 
}

void  MCPresetPin(byte  adres, byte  pinnummer ){
 // Uitzetten van een pin van de MCP23017
 // pinnummers van groep A en B zijn doorgenummerd als 0 tm 15
 double pindouble = 0;
 byte pinwaarde = 0;
 byte oldpinwaarde = 0;
 byte GPIO = 0;
 if (pinnummer < 8){
  GPIO = 0x12;                      //select GPIOA pins 0-7
  pindouble = round(pow(2,pinnummer));     //omzetten naar bitwaarde 
 }else{  
  GPIO = 0x13;                      //select GPIOB pins 0-7
  pindouble = round(pow(2,pinnummer-8));   //omzetten naar bitwaarde 
 } 
 Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(GPIO);                  //GPIOA of GPIOB 
 Wire.endTransmission();            //stop commmunicatie met MCP 
 Wire.requestFrom(adres, 0x1);
 oldpinwaarde = Wire.read();        //haal huidige pinwaarde op 
 pinwaarde = pindouble; 
 pinwaarde = (oldpinwaarde & ~pinwaarde);
  Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(GPIO);                  //GPIOA of GPIOB 
 Wire.write(pinwaarde);             //set pin 
 Wire.endTransmission();            //stop commmunicatie met MCP 
}

void  MCPflipPin(byte adres, byte pinnummer){
 // Wisselen van een pin van de MCP23017
 // pinnummers van groep A en B zijn doorgenummerd als 0 tm 15
 double pindouble = 0;
 byte pinwaarde = 0;
 byte oldpinwaarde = 0;
 byte GPIO = 0;
 if (pinnummer < 8){
  GPIO = 0x12;                      //select GPIOA pins 0-7
  pindouble = round(pow(2,pinnummer));      //omzetten naar bitwaarde 
 }else{  
  GPIO = 0x13;                      //select GPIOB pins 0-7
  pindouble = round(pow(2,pinnummer-8));    //omzetten naar bitwaarde
 } 
 Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(GPIO);                  //GPIOA of GPIOB 
 Wire.endTransmission();            //stop commmunicatie met MCP 
 Wire.requestFrom(adres, 0x1);
 oldpinwaarde = Wire.read();        //haal huidige pinwaarde op 
 pinwaarde = pindouble;
 pinwaarde = (oldpinwaarde ^pinwaarde);
 Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(GPIO);                  //GPIOA of GPIOB 
 Wire.write(pinwaarde);             //set pin 
 Wire.endTransmission();            //stop commmunicatie met MCP 
}

bool  MCPreadPin(byte adres, byte pinnummer){
 // Uitlezen van een pin van de MCP23017
 // NB, ook de status van outputpinnen kan worden uitgelezen
 // pinnummers van groep A en B zijn doorgenummerd als 0 tm 15
 double pindouble = 0;
 byte pinwaarde = 0;
 byte oldpinwaarde = 0;
 byte GPIO = 0;
 if (pinnummer < 8){
  GPIO = 0x12;                      //select GPIOA pins 0-7
  pindouble = round(pow(2,pinnummer));   //omzetten naar bitwaarde 
 }else{  
  GPIO = 0x13;                      //select GPIOB pins 0-7
  pindouble = round(pow(2,pinnummer-8)); //omzetten naar bitwaarde
 } 
 Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(GPIO);                  //GPIOA of GPIOB 
 Wire.endTransmission();            //stop commmunicatie met MCP 
 Wire.requestFrom(adres, 0x1);
 oldpinwaarde = Wire.read();        //haal huidige pinwaarde op 
 pinwaarde = pindouble;
 pinwaarde = (oldpinwaarde & pinwaarde);
 return pinwaarde; 
}

void MCPreadAllpins(byte adres, byte &byteA, byte &byteB){
 // Uitlezen van alle pinnen van de MCP23017
 // NB, ook de status van outputpinnen kan worden uitgelezen
 // byteA en byteB bevatten de som van de pinwaarden van de pinnen
 Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(0x12);                  //GPIOA of GPIOB 
 Wire.endTransmission();            //stop commmunicatie met MCP 
 Wire.requestFrom(adres, 0x1);       //een byte   
 byteA = Wire.read();               //haal huidige pinwaarden op 
 Wire.beginTransmission(adres);     //begin communicatie met MCP
 Wire.write(0x13);                  //GPIOA of GPIOB 
 Wire.endTransmission();            //stop commmunicatie met MCP 
 Wire.requestFrom(adres, 0x1);       //een byte
 byteB = Wire.read();               //haal huidige pinwaarden op  
}

// MCP23017 routines
//------------------
void MCPinit(byte adres, byte Afunctie, byte Bfunctie) { 
 // Initialiseren van alle pinnen van de MCP23017 
 // NB hoeft maar eenmalig, of als pinnen aan andere functie krijgen.
 // Adres opgeven en pinnen instellen voor input of output
 // Het adres van de MCP is hex 20tm27, afhankelijk van adresjumpers
 //     jumper A0 telt voor 1, A1 voor 2 en A2 voor 4.
 // Afunctie en Bfunctie zijn de functiebits voor de poorten A en B
 // Als een pin output moet zijn staat de bit uit
 // Als een pin input moet zijn staat de bit aan
 // Bijv:   MCPinit( 0x20, B00000001, B11111110) 
 //               poort A pin 7 tm 1 output, pin0 input
 //               poort B pin 7 tm 1 input,  pin0 output
 // Dit kan je ook opgeven als:
 //         MCPinit( 0x20, 1, 254)
 byte byta = 0;
 byte bytb = 0; 
 Wire.begin();                    //create Wire object
 Wire.beginTransmission(adres);   //begin communicatie met MCP
 Wire.write(byte(0));             //select IODIRA register, ports 0-7
 Wire.write(Afunctie);            //alle ports 0-7 outputs
 Wire.endTransmission();          //stop commmunicatie met MCP
 Wire.beginTransmission(adres);   //begin communicatie met MCP
 Wire.write(0x01);                //select IODIRB register, ports 8-15
 Wire.write(Bfunctie);            //alle ports 8-15 outputs
 Wire.endTransmission();          //stop commmunicatie met MCP

 //MCPsetAllpins(adres,127,127);    //test of unit ready is 127 pinnen
 //MCPreadAllpins(adres,byta,bytb); 
 //if((byta !=127) || (bytb !=127)){
 MCPsetAllpins(adres,1,1);        //test of unit ready is met 1 pin
 MCPreadAllpins(adres,byta,bytb); 
 if((byta !=1) || (bytb !=1)){

  MCPstatus = 9;                  //unit niet ready
 }else{
  MCPstatus = 1;                  //OK, ready                
 } 
 MCPsetAllpins(adres,0,0);        //reset buffer tbv volgende init 
}

