#ifndef ESP32
 FOUT alleen voor ESP32
#endif

File     MIDfileSP;

void MidiwriteheaderSP(String filenm, long &trackleng){
 // Vaste header met tempo MM100
 // Deltatime 100 = 500 millisec
 // LET op: Arduino werkt met big endian, midi met little endian.
 //         Direct schrijven van integers kan dus niet.
 //         Waarden worden daarom per byte weggeschreven, 
 //         de high order bytes eerst.
 byte b3;
 byte b2;
 byte b1;
 long w = 0;
 String filenaam = "/"+filenm;
 if (SD.exists(filenaam.c_str())) {
  SD.remove(filenaam.c_str());
 }
 MIDfileSP = SPIFFS.open(filenaam, FILE_WRITE);
 if (!MIDfileSP) {
   return;
 }    
 MIDfileSP.print("MThd");       //00 Midi header   
 MIDfileSP.write(byte(0));      //04 headerlengte 6 (4bytes)    
 MIDfileSP.write(byte(0));       
 MIDfileSP.write(byte(0));       
 MIDfileSP.write(0x06);  
 MIDfileSP.write(byte(0));      //08 format 0 (2bytes)    
 MIDfileSP.write(byte(0));                   
 MIDfileSP.write(byte(0));      //10 aant tracks 1 (2bytes)       
 MIDfileSP.write(0x01);                                        
 MIDfileSP.write(byte(0));      //12 tickperkwart 120 (2bytes)       
 MIDfileSP.write(byte(120));                                      
 
 MIDfileSP.print("MTrk");       //14 Midi track    
 MIDfileSP.write(byte(0));      // 18(4) trackleng, ex MTrk en leng
 MIDfileSP.write(byte(0));      //Wordt na schrijven 
 MIDfileSP.write(byte(0));      //van de file 
 MIDfileSP.write(byte(0));      //ingevuld
 MIDfileSP.write(byte(0));      //22 1e delta etc
 MIDfileSP.write(0xFF);         //meta             
 MIDfileSP.write(0x59);         //keysign         
 MIDfileSP.write(0x02);        
 MIDfileSP.write(byte(0));         
 MIDfileSP.write(byte(0));   
 MIDfileSP.write(byte(0));      //delta
 MIDfileSP.write(0xFF);         //meta
 MIDfileSP.write(0x51);         //tempo
 MIDfileSP.write(0x03);         //3 bytes lengte tempo   
 w = 600000;                  //tempo 600000 (mm=100) hex0927C0       
 b1 = ((w>>(0)) & 0xff);      //low order byte
 b2 = ((w>>(8)) & 0xff);
 b3 = ((w>>(16)) & 0xff);     //high order byte   
 MIDfileSP.write(byte(b3));
 MIDfileSP.write(byte(b2));
 MIDfileSP.write(byte(b1));
 MIDfileSP.write(byte(0));      //delta 
 MIDfileSP.write(0xFF);         //meta
 MIDfileSP.write(0x58);         //timesign      
 MIDfileSP.write(0x04);         
 MIDfileSP.write(0x04);          
 MIDfileSP.write(0x02);          
 MIDfileSP.write(0x18);        
 MIDfileSP.write(0x08);  
 trackleng = 21;
}

void MidiwriteEventSP(byte mes,byte data1, byte data2, 
                    long deltatime, long &trackleng){
 // Schrijf midi-event
 byte b1 = 0;
 byte b2 = 0;
 byte b3 = 0;
 byte b4 = 0;
 long dt = deltatime/5;  //millisec naar ticks bij mm100
 // deltatime is 1 tot 4 bytes lang. 7-bits notatie. De meest rechtse
 //           byte heeft high order bit 0, eventuele extra bytes
 //           hebben de high order bit 1. 
 b4 = dt%128; dt = dt / 128;  //meest rechtse, high order bit 0 
 b3 = dt%128; b3 = b3 + 128; dt = dt / 128;  //high order bit 1
 b2 = dt%128; b2 = b2 + 128; dt = dt / 128;  //high order bit 1
 b1 = dt%128; b1 = b1 + 128;                 //high order bit 1
 if (b1 > 128){
  MIDfileSP.write(b1); MIDfileSP.write(b2); MIDfileSP.write(b3); 
  trackleng = trackleng + 4;   //4 bytes
 }else{
  if (b2 > 128){
   MIDfileSP.write(b2); MIDfileSP.write(b3); 
   trackleng = trackleng + 3;  //3 bytes
  }else{
   if (b3 > 128){
    MIDfileSP.write(b3); 
    trackleng = trackleng + 2; //2 bytes
   }else{
    trackleng = trackleng+1;   //1 bytes
   }
  }  
 }
 MIDfileSP.write(b4); 
 MIDfileSP.write(mes);
 MIDfileSP.write(data1);
 MIDfileSP.write(data2); 
 trackleng = trackleng+ 3; 
}
void MidiwriteEOFSP(long &trackleng){
 byte b4;
 byte b3;
 byte b2;
 byte b1;
 long w = 0;
 MIDfileSP.write(byte(0));      //delta
 MIDfileSP.write(0xFF);         //FF2F  = EOT
 MIDfileSP.write(0x2F);
 MIDfileSP.write(byte(0)); 
 trackleng = trackleng +4; 
 MIDfileSP.seek (18, fs::SeekSet);      //positioneer na MTrk, positie 19
           // was seekset, nog testen
 w = trackleng;               //tracklengte
 b1 = ((w>>(0)) & 0xff);      //low order byte
 b2 = ((w>>(8)) & 0xff);
 b3 = ((w>>(16)) & 0xff);      
 b4 = ((w>>(24)) & 0xff);     //high order byte   
 MIDfileSP.write(byte(b4));
 MIDfileSP.write(byte(b3));
 MIDfileSP.write(byte(b2));
 MIDfileSP.write(byte(b1));     //tracklengte 4 bytes    
 MIDfileSP.close();  
} 