
#ifdef ESP32
 #include "SPIFFS.h"
#else
 FOUT alleen voor ESP32
#endif
File   spfile;  
long   SPIFFfileinx = 0;

// SPIFFS routines
//------------------
void SPinit() {             //Start SPIFFS
 SPunitstatus = 1;          //SPIFF beschikbaar   
 if(!SPIFFS.begin()){ 
  SPunitstatus = 9;
  return;
 }
}

void SPformat() {           //Format SPIFFS
 SPunitstatus = 1;          //Format OK, SPIFF beschikbaar   
 if(!SPIFFS.format()){ 
  SPunitstatus = 9;
  return;
 }
}

long SPtotal(){
 //returns totale ruimte in de partitie
 return SPIFFS.totalBytes();
}

long SPused(){
 //returns gebruikte ruimte in de partitie
 return SPIFFS.usedBytes();
}

void SPopenDirectory(String extensie) {
 // lees alle filenames in de directory en sla op in array SPfilenaam. 
 String filenaam = "";
 String ext  = extensie;
 ext.toUpperCase();
 String fil= "";
 int swselectfile = 0;
 const char* fileentry ;
 SPfilecount = 0;
 SPfileinx = 0;
 SPdirstatus = 1;
 for (int i=1;i<=SPfilemax;i++){  
  SPfilenaam[i] = ""; 
 } 
 File spdir = SPIFFS.open("/"); // Open dir folder

 while (true) {
  File entry =  spdir.openNextFile();
  if (! entry) {  
   break;                      //EOF
  }
  if (SPfileinx == SPfilemax){  //indexen begint met 1
   SPdirstatus = 2;   //Open OK, maar teveel files in dir
   return;
  }
  fileentry = entry.name(); 
  fil = fileentry;
  fil.toUpperCase();
  if (extensie ==""){
   swselectfile = 1;
  }else{
   if (fil.indexOf(ext)> -1){
    swselectfile = 1;
   }else{
    swselectfile = 0;
   }
  }

  if (swselectfile == 1){
   SPfileinx ++;
   SPfilecount ++;
   filenaam = fileentry;
   filenaam.replace("/", "");
   SPfilenaam[SPfileinx] = filenaam;
   if (SPfilenaam[SPfileinx] == filenaam){ 
   }else{                //plaatsen in array mislukt
    SPdirstatus = 3;     //Out of memory, namen in dir te lang, array vol
    return;
   }
   if (SPfilenaam[SPfileinx] == ""){  //plaatsen in array mislukt               
    SPdirstatus = 3;     //Out of memory, namen in dir te lang, array vol
    return;
   }
  }
  entry.close();
 }    // end while
}

void SPcreatefile(String filenm){
 //Maak een lege file aan. Als deze al bestand wordt deze eerst
 //deleted
 String filenaam = "/"+ filenm;
 if (SPIFFS.exists(filenaam.c_str())) {
  SPIFFS.remove(filenaam.c_str());
 }
 File file = SPIFFS.open(filenaam, FILE_WRITE);
 if(!file){
  Serial.println("Error opening file for writing");
  return;
 } 
 file.close(); 
}

void SPdeletefile(String filenm){
 //Delete file
 String filenaam = "/"+ filenm;
 SPIFFS.remove(filenaam.c_str());
}

void SPopenfileread(String filenm){
 //open file voor lezen 
 String filenaam = "/"+ filenm;
 if (filenaam == ""){
  SPfilestatus = 8;
  return;
 } 
 if (!SPIFFS.exists(filenaam.c_str())) {
  SPfilestatus = 8;
  return;
 }
 spfile = SPIFFS.open(filenaam);
 if (spfile) {
  SPfilestatus = 1;
 }else{
  SPfilestatus = 8;
 }
}

void SPseek(long positie){
 //zet positie van volgende IO
 //NB, de eerste byte in de file is 0
 spfile.seek(positie, fs::SeekSet);
}

long SPposition(){
 //returns fileposition van open file,
 //dat is het nummer van de volgende IO
 //NB, de eerste byte in de file is 0
 return spfile.position();
}

void SPopenwrite(String filenm){
 //Open file voor write. Als de file niet bestond
 //wordt een lege aangemaakt.
 String filenaam = "/"+ filenm;
 if (filenaam == ""){
  SPfilestatus = 8;
  return;
 } 
 if (SPIFFS.exists(filenaam.c_str())) {
  SPIFFS.remove(filenaam.c_str());
 }
 spfile = SPIFFS.open(filenaam, FILE_WRITE);
 if (spfile) {
  SPfilestatus = 2;
 }else{
  SPfilestatus = 8;
 }
}

void SPopenappend(String filenm){
 //Open file voor append. De hierna geschreven data
 //wordt achteraan toegevoegd.  
 String filenaam = "/"+ filenm;
 if (filenaam == ""){
  SPfilestatus = 8;
  return;
 } 
 #ifdef ESP32
  spfile = SPIFFS.open(filenaam, FILE_APPEND);
 #else
  spfile = SPIFFS.open(filenaam,  O_APPEND|O_WRITE);
 #endif
 if (spfile) {
  SPfilestatus = 2;
 }else{
  SPfilestatus = 8;
 }
}

void SPclosefile(){
 //close file
 //NB: als je na schijven geen close doet wordt de filesize niet
 //    bijgewerkt en gaan de schfijfopdrachten verloren.
 //    Zowel bij write als append.
 spfile.close();
 SPfilestatus = 0; 
}

void SPprintdirentries(String dir){
 // Print directory met subdirs (recursieve call)
 int lasti = 0;
 String lastdir = "";
 String filenaam = "";
 SPopenDirectory("");
 Sortstring(SPfilenaam, 1, SPfilecount,0);
 for (int i = 1; i <= SPfilecount; i++) {
  filenaam = SPfilenaam[i];
  Serial.println("   "+ filenaam);
 } 
}



long SPfilesize(){
 //returns file size van open file
 return spfile.size();
}


bool SPavailable(){
 //returns availability van open file
 return spfile.available();
}

void SPreadline(String &record){
 //lees een lijn, alleen asci chars, CR is end of line 
 if (SPfilestatus != 1){
  return;
 }
 record = "";
 while (spfile.available()) { //tot end of file
  char Inbyte = spfile.read();
  if (Inbyte == '\r') {       //CR is end of record
   return;                    //record gelezen
  }else{ 
   if (Inbyte > 31) {         //asci character
    record += Inbyte;         //byte in record
   }                          //en leees volgende
  }  
 }
 spfile.close();
 SPfilestatus = 5;            //end of file
}

void SPprintline(String line){
 //Schrijf een line, beeindigd met CRLF 
 spfile.println(line);
}


void SPreadByte(byte &databyte){
 databyte = spfile.read();      
}
void SPreadChar(char &databyte){
 databyte = spfile.read();      
}
void SPreadInt(int &dataint){
  int i2 = spfile.read();    //low order (big endian)
  int i1 = spfile.read();    //high order
  dataint = ((i2 << 0) & 0xFF) + ((i1 << 8) & 0xFFFF);
}
void SPreadIntDOS(int &dataint){
  int i1 = spfile.read();    //high order (little endian)
  int i2 = spfile.read();    //low order
  dataint = ((i2 << 0) & 0xFF) + ((i1 << 8) & 0xFFFF);
}
void SPreadLong(long &datalong){
 long l4 = spfile.read();   //low order (big endian)
 long l3 = spfile.read();
 long l2 = spfile.read();
 long l1 = spfile.read();   //high order
 datalong = ((l4 << 0) & 0xFF) + ((l3 << 8) & 0xFFFF)
          + ((l2 << 16) & 0xFFFFFF) + ((l1 << 24) & 0xFFFFFFFF);        
}
void SPreadLongDOS(long &datalong){
 long l1 = spfile.read();   //high order  (little endian)
 long l2 = spfile.read();
 long l3 = spfile.read();
 long l4 = spfile.read();   //low order
 datalong = ((l4 << 0) & 0xFF) + ((l3 << 8) & 0xFFFF)
          + ((l2 << 16) & 0xFFFFFF) + ((l1 << 24) & 0xFFFFFFFF);        
}
void SPreadLong3DOS(long &datalong){
 long l1 = spfile.read();   //high order  (3 bytes little endian)
 long l2 = spfile.read();
 long l3 = spfile.read();   //low order
 datalong = ((l3 << 0) & 0xFF) + ((l2 << 8) & 0xFFFF)
          + ((l1 << 16) & 0xFFFFFF) ;        
}
  
void SPwriteByte(byte databyte){
 spfile.write(databyte);      
}
void SPwriteChar(char databyte){
 spfile.write(databyte);      
}

void SPwriteInt(int dataint){
 byte b2 = (dataint & 0xFF);         //low order 
 byte b1 = ((dataint >> 8) & 0xFF);  //high order
 spfile.write(b2);  //low order (big endian)
 spfile.write(b1);  //high order
}
void SPwriteIntDOS(int dataint){
 byte b1 = (dataint & 0xFF);         //low order (big endian)
 byte b2 = ((dataint >> 8) & 0xFF);  //high order
 spfile.write(b2);  //high order (little endian)
 spfile.write(b1);  //low order
}
void SPwriteLong(long datalong){
 byte b4 = (datalong & 0xFF);         //low order 
 byte b3 = ((datalong >> 8) & 0xFF);
 byte b2 = ((datalong >> 16) & 0xFF);
 byte b1 = ((datalong >> 24) & 0xFF); //high order
 spfile.write(b4);  //low order (big endian)
 spfile.write(b3);
 spfile.write(b2);
 spfile.write(b1);  //high order
}
void SPwriteLongDOS(long datalong){
 byte b1 = (datalong & 0xFF);         //low order 
 byte b2 = ((datalong >> 8) & 0xFF);
 byte b3 = ((datalong >> 16) & 0xFF);
 byte b4 = ((datalong >> 24) & 0xFF); //high order
 spfile.write(b4);  //high order (little endian)
 spfile.write(b3);
 spfile.write(b2);
 spfile.write(b1);  //low order
}


