#ifdef ESP32
 #include "ESP32TimerInterrupt.h"   //Timer interrupt library voor ESP's
 ESP32Timer Timer0(0);              //maak instance
#endif
#ifdef AVR
 #include <TimerOne.h>              //Timer interrupt library voor Arduino's
#endif 
#ifdef ESP8266
#include "ESP8266TimerInterrupt.h"   //Timer interrupt library voor ESP's
 ESP8266Timer Timer0;                //maak instance
#endif

// Timer routines
//------------------
volatile int TIMsw = 0;  //de timer switch wordt gezet door deze interrupt routine
// deze ISR wordt gecall-back't als de Timer  afloopt

#ifdef ESP8266
 IRAM_ATTR void TimerInterrupt(){  
 TIMsw ++;  
}           
#endif
#ifdef ESP32
void IRAM_ATTR TimerInterrupt(void) { 
 TIMsw ++;   
}    
#endif
#ifdef AVR
void TimerInterrupt(){  
 TIMsw ++;  
}           
#endif

void TIMERinit(){
 // wordt eenmalig tijdens setup gedaan  
#ifdef ESP32
 TIMsw = 0;                              
 Timer0.attachInterrupt(10,TimerInterrupt);  //wijs ISR aan, en start timer met  
 Timer0.stopTimer();                         //dummy interval, dus stop weer
#endif
#ifdef AVR
 TIMsw = 0;                         
 Timer1.initialize();              
 Timer1.attachInterrupt(TimerInterrupt,1000);  //wijs ISR aan, timer staat stil
#endif 
#ifdef ESP8266
 TIMsw = 0;                              
 Timer0.attachInterrupt(10,TimerInterrupt);  //wijs ISR aan, en start timer met  
 Timer0.stopTimer();                         //dummy interval, dus stop weer
#endif
}
void TIMERstart(long microsecs){   //start timer met nummer 
 if (microsecs == 0){              //geen timer nodig
  TIMsw = 3;                       //timer loopt nu direct af
  return;
 }
#ifdef ESP32 
 TIMsw = 2;                        //ESP geeft eerste interrupt pas als timer afloopt
 Timer0.setInterval(microsecs,TimerInterrupt);  
 Timer0.restartTimer();                         
#endif 
#ifdef AVR
 TIMsw = 1;                        //Arduino heeft extra interrupt bij start timer   
 Timer1.setPeriod(microsecs);
 Timer1.start();
#endif
#ifdef ESP8266
 TIMsw = 2;                        //ESP geeft eerste interrupt pas als timer afloopt
 Timer0.setInterval(microsecs,TimerInterrupt);  
#endif
}

bool TIMERhasinterrupted(){        //uitvragen met "if(TIMERhasinterrupted")
 if (TIMsw == 0){
  return (true);     //timer was niet gestart of al afgelopen
 }     
#ifdef ESP32
 if (TIMsw >= 3){
  Timer0.stopTimer();       
  TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERfirstinterrupted
  TIMsw = 0;   
  return(true);
 }else{ 
  return(false); 
 }
#endif 
#ifdef AVR  
 if (TIMsw >= 3){          
  Timer1.stop();        
  TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERfirstinterrupted
  TIMsw = 0;    
  return(true);
 }else{
  return(false); 
 }  
#endif  
#ifdef ESP8266
 if (TIMsw >= 3){
  Timer0.stopTimer();       
  TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERfirstinterrupted
  TIMsw = 0;   
  return(true);
 }else{ 
  return(false); 
 }
#endif

}

void TIMERwait(){         //wacht tot timer afgelopen is
 if (TIMsw == 0){return ;} //timer was niet gestart of al afgelopen
#ifdef ESP32
 while (TIMsw < 3){;}     //timer loop af als ISR TIMsw tot 3 heeft opgehoogd 
 Timer0.stopTimer();   
 TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERwait
 TIMsw = 0;
 }
#endif
#ifdef AVR 
 while (TIMsw < 3){;}     //timer loop af als ISR TIMsw tot 3 heeft opgehoogd       
 Timer1.stop();
 TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERwait
 TIMsw = 0;        
 }
#endif  
#ifdef ESP8266
 while (TIMsw < 3){;}     //timer loop af als ISR TIMsw tot 3 heeft opgehoogd 
 Timer0.stopTimer();   
 TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERwait
 TIMsw = 0;
}
#endif


void TIMERreset(){       //reset timer zodat TIMERwait en TIMERfirstinterrupted doorlopen
#ifdef ESP32
 Timer0.stopTimer();   
 TIMsw = 0;
#endif 
#ifdef AVR 
 Timer1.stop();
 TIMsw = 0;             
#endif  
#ifdef ESP8266
 Timer0.stopTimer();   
 TIMsw = 0;
#endif
}

#ifdef ESP32
 void TIMERdelay(long microsecs){  //wacht aantal microseconden
 if (microsecs == 0){             //timer loopt direct af
  return;
 }
 TIMsw = 2;                      //ESP geeft eerste interrupt pas als timer afloopt
 Timer0.setInterval(microsecs,TimerInterrupt); 
 Timer0.restartTimer();  
 while(true){
  if (TIMsw >= 3){               //teller wordt opgehoogd door ISR
   Timer0.stopTimer();           //timer afgelopen 
   break;
  }
 }  
 TIMsw = 0;
 }                     
#endif
#ifdef AVR
void TIMERdelay(long microsecs){  //wacht aantal microseconden
 if (microsecs == 0){             //timer loopt direct af
  return;
 }
 TIMsw = 1;                      //Arduino heeft extra interrupt bij start timer   
 Timer1.setPeriod(microsecs);
 Timer1.start();
 while(true){
  if (TIMsw >= 3){               //teller wordt opgehoogd door ISR
   Timer1.stop();                //timer afgelopen 
   break;
  }
 }
 TIMsw = 0;
}
#endif 
#ifdef ESP8266
 void TIMERdelay(long microsecs){  //wacht aantal microseconden
 if (microsecs == 0){              //timer loopt direct af
  return;
 }
 if (microsecs < 16000){
  delayMicroseconds (microsecs) ; //geeft crash
 }else{
  delay(microsecs/1000);
 }
 //            Code hieronder crasht ook
 //TIMsw = 2;                      //ESP geeft eerste interrupt pas als timer afloopt
 //Timer0.setInterval(microsecs,TimerInterrupt); 
 //while(true){
 // if (TIMsw >= 3){               //teller wordt opgehoogd door ISR
 //  Timer0.stopTimer();           //timer afgelopen 
 //  TIMsw = 0;
 //  return;
 // }
 //}  
 }                     
#endif
