
#ifdef ESP32
 #include "ESP32TimerInterrupt.h"   //Timer interrupt library voor ESP's
 ESP32Timer Timer0(0);              //maak instance
#else
 #include <TimerOne.h>              //Timer interrupt library voor Arduino's
#endif 

// Timer routines
//------------------
volatile int TIMsw = 0;  //de timer switch wordt gezet door deze interrupt routine
// deze ISR wordt gecall-back't als de Timer  afloopt
#ifdef ESP32
void IRAM_ATTR TimerInterrupt(void) { 
 TIMsw ++;   
}    
#else
void TimerInterrupt(){  
 TIMsw ++;  
}           
#endif

void TIMERinit(){
 // wordt eenmalig tijdens setup gedaan  
#ifdef ESP32
 TIMsw = 0;                              
 Timer0.attachInterrupt(10,TimerInterrupt);  //wijs ISR aan, en start timer met  
 Timer0.stopTimer();                         //dummy interval, dus stop weer
#else
 TIMsw = 0;                         
 Timer1.initialize();              
 Timer1.attachInterrupt(TimerInterrupt,1000);  //wijs ISR aan, timer staat stil
#endif 
}
void TIMERstart(long microsecs){   //start timer met nummer 
 if (microsecs == 0){              //geen timer nodig
  TIMsw = 3;                       //timer loopt nu direct af
  return;
 }
#ifdef ESP32 
 TIMsw = 2;                        //ESP geeft eerste interrupt pas als timer afloopt
 Timer0.setInterval(microsecs,TimerInterrupt);  
 Timer0.restartTimer();                         
#else 
 TIMsw = 1;                        //Arduino heeft extra interrupt bij start timer   
 Timer1.setPeriod(microsecs);
 Timer1.start();
#endif
}

bool TIMERhasinterrupted(){        //uitvragen met "if(TIMERhasinterrupted")
 if (TIMsw == 0){
  return (true);     //timer was niet gestart of al afgelopen
 }     
#ifdef ESP32
 if (TIMsw >= 3){
  Timer0.stopTimer();       
  TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERfirstinterrupted
  TIMsw = 0;   
  return(true);
 }else{ 
  return(false); 
 }
#else   
 if (TIMsw >= 3){          
  Timer1.stop();        
  TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERfirstinterrupted
  TIMsw = 0;    
  return(true);
 }else{
  return(false); 
 }  
#endif  
}

void TIMERwait(){         //wacht tot timer afgelopen is
 if (TIMsw == 0){return ;} //timer was niet gestart of al afgelopen
#ifdef ESP32
 while (TIMsw < 3){;}     //timer loop af als ISR TIMsw tot 3 heeft opgehoogd 
  Timer0.stopTimer();   
  TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERwait
  TIMsw = 0;
 }
#else   
 while (TIMsw < 3){;}     //timer loop af als ISR TIMsw tot 3 heeft opgehoogd       
  Timer1.stop();
  TIMERexceed = TIMsw - 3; //timercycles die extra verstreken zijn voor TIMERwait
  TIMsw = 0;        
 }
#endif  

void TIMERreset(){       //reset timer zodat TIMERwait en TIMERfirstinterrupted doorlopen
#ifdef ESP32
 Timer0.stopTimer();   
 TIMsw = 0;
#else  
 Timer1.stop();
 TIMsw = 0;             
#endif  
}

#ifdef ESP32
 void TIMERdelay(long microsecs){  //wacht aantal microseconden
 if (microsecs == 0){             //timer loopt direct af
  return;
 }
 TIMsw = 2;                      //ESP geeft eerste interrupt pas als timer afloopt
 Timer0.setInterval(microsecs,TimerInterrupt); 
 Timer0.restartTimer();  
 while(true){
  if (TIMsw >= 3){               //teller wordt opgehoogd door ISR
   Timer0.stopTimer();           //timer afgelopen 
   break;
  }
 }  
 TIMsw = 0;
 }                     
#else
void TIMERdelay(long microsecs){  //wacht aantal microseconden
 if (microsecs == 0){             //timer loopt direct af
  return;
 }
 TIMsw = 1;                      //Arduino heeft extra interrupt bij start timer   
 Timer1.setPeriod(microsecs);
 Timer1.start();
 while(true){
  if (TIMsw >= 3){               //teller wordt opgehoogd door ISR
   Timer1.stop();                //timer afgelopen 
   break;
  }
 }
 TIMsw = 0;
}
#endif 